@@ -18,6 +18,15 @@ var ebi = document.getElementById.bind(document),
     mknod = document.createElement.bind(document);
 
 
+function qsr(sel) {
+    var el = QS(sel);
+    if (el)
+        el.parentNode.removeChild(el);
+
+    return el;
+}
+
+
 // error handler for mobile devices
 function esc(txt) {
     return txt.replace(/[&"<>]/g, function (c) {
@@ -71,7 +80,7 @@ try {
 catch (ex) {
     if (console.stdlog)
         console.log = console.stdlog;
-    console.log(ex);
+    console.log('console capture failed', ex);
 }
 var crashed = false, ignexd = {};
 function vis_exh(msg, url, lineNo, columnNo, error) {
@@ -163,7 +172,6 @@ function vis_exh(msg, url, lineNo, columnNo, error) {
     catch (e) {
         document.body.innerHTML = html.join('\n');
     }
-    throw 'fatal_err';
 }
 function ignex(all) {
     var o = ebi('exbox');
@@ -727,7 +735,7 @@ function hist_replace(url) {
 
 function sethash(hv) {
     if (window.history && history.replaceState) {
-        hist_replace(document.location.pathname + '#' + hv);
+        hist_replace(document.location.pathname + document.location.search + '#' + hv);
     }
     else {
         document.location.hash = hv;
@@ -1057,15 +1065,22 @@ var modal = (function () {
     }
 
     function onkey(e) {
-        if (e.code == 'Enter') {
-            var a = ebi('modal-ng');
-            if (a && document.activeElement == a)
+        var k = e.code,
+            eok = ebi('modal-ok'),
+            eng = ebi('modal-ng'),
+            ae = document.activeElement;
+
+        if (k == 'Space' && ae && (ae === eok || ae === eng))
+            k = 'Enter';
+
+        if (k == 'Enter') {
+            if (ae && ae == eng)
                 return ng();
 
             return ok();
         }
 
-        if (e.code == 'Escape')
+        if (k == 'Escape')
             return ng();
     }
 
@@ -1142,6 +1157,7 @@ function repl_load() {
         if (!ret.length)
             ret = [
                 'var v=Object.keys(localStorage); v.sort(); JSON.stringify(v)',
+                "for (var a of QSA('#files a[id]')) a.setAttribute('download','')",
                 'console.hist.slice(-10).join("\\n")'
             ];
 
@@ -1213,28 +1229,31 @@ if (ebi('repl'))
     ebi('repl').onclick = repl;
 
 
+var svg_decl = '<?xml version="1.0" encoding="UTF-8"?>\n';
+
+
 var favico = (function () {
     var r = {};
     r.en = true;
+    r.tag = null;
 
     function gx(txt) {
-        return (
-            '<?xml version="1.0" encoding="UTF-8"?>\n' +
-            '<svg version="1.1" viewBox="0 0 64 64" xmlns="http://www.w3.org/2000/svg"><g>\n' +
+        return (svg_decl +
+            '<svg version="1.1" viewBox="0 0 64 64" xmlns="http://www.w3.org/2000/svg">\n' +
             (r.bg ? '<rect width="100%" height="100%" rx="16" fill="#' + r.bg + '" />\n' : '') +
             '<text x="50%" y="55%" dominant-baseline="middle" text-anchor="middle"' +
             ' font-family="sans-serif" font-weight="bold" font-size="64px"' +
-            ' fill="#' + r.fg + '">' + txt + '</text></g></svg>'
+            ' fill="#' + r.fg + '">' + txt + '</text></svg>'
         );
     }
 
-    r.upd = function () {
-        var i = QS('link[rel="icon"]'), b64;
+    r.upd = function (txt, svg) {
         if (!r.txt)
             return;
 
+        var b64;
         try {
-            b64 = btoa(gx(r.txt));
+            b64 = btoa(svg ? svg_decl + svg : gx(r.txt));
         }
         catch (ex) {
             b64 = encodeURIComponent(r.txt).replace(/%([0-9A-F]{2})/g,
@@ -1243,12 +1262,12 @@ var favico = (function () {
             b64 = btoa(gx(unescape(encodeURIComponent(r.txt))));
         }
 
-        if (!i) {
-            i = mknod('link');
-            i.rel = 'icon';
-            document.head.appendChild(i);
+        if (!r.tag) {
+            r.tag = mknod('link');
+            r.tag.rel = 'icon';
+            document.head.appendChild(r.tag);
         }
-        i.href = 'data:image/svg+xml;base64,' + b64;
+        r.tag.href = 'data:image/svg+xml;base64,' + b64;
     };
 
     r.init = function () {
